import torch
from scipy.sparse.linalg import eigs
from scipy.sparse.linalg import LinearOperator

def dominant_eigensolver(Hopt, v0, tol):
    N = v0.shape[0]
    A = LinearOperator((N, N), matvec=lambda x: Hopt(torch.as_tensor(x)).detach().numpy())
    vals, vecs = eigs(A, k=1, which='LM', v0=v0.detach().numpy(), tol=tol)

    w = torch.as_tensor(vals[0].real, dtype=v0.dtype, device=v0.device)
    v = torch.as_tensor(vecs[:, 0].real, dtype=v0.dtype, device=v0.device)
    return w, v

